package grapher.ui;

import java.awt.Dimension;
import javax.swing.JList;
import javax.swing.JSplitPane;
import javax.swing.JMenuBar;
import javax.swing.JMenu;

import javax.swing.Action;
import javax.swing.AbstractAction;
import javax.swing.JOptionPane;

import java.awt.Color;
import javax.swing.table.DefaultTableModel;

import java.awt.Toolkit;
import javax.swing.KeyStroke;

import java.awt.event.*;
import javax.swing.event.*;

import grapher.fc.*;


public class FunctionPane extends JSplitPane {
	protected DefaultTableModel functions;
	
	protected Action creater, remover;
	
	protected ListPane list;
	protected Grapher graph;
	
	public FunctionPane(JMenuBar menubar) {
		super(HORIZONTAL_SPLIT);
		setContinuousLayout(true);
		
		functions = new DefaultTableModel(0, 2);
		
		// actions
		final int modifier = Toolkit.getDefaultToolkit().getMenuShortcutKeyMask();
		creater = new AbstractAction(" + ") {
			public void actionPerformed(ActionEvent event) { create_function(); }};
		final int create_key = KeyEvent.VK_N;
		creater.putValue(Action.ACCELERATOR_KEY, 
		               KeyStroke.getKeyStroke(create_key, modifier));
		creater.putValue(Action.MNEMONIC_KEY, create_key);

		remover = new AbstractAction(" - ") {
			public void actionPerformed(ActionEvent e)     { remove_functions(); }};
		final int remove_key = KeyEvent.VK_BACK_SPACE;
		remover.putValue(Action.ACCELERATOR_KEY, 
		                 KeyStroke.getKeyStroke(remove_key, modifier));
		remover.putValue(Action.MNEMONIC_KEY, remove_key);
				
		// expression menu
		JMenu menu = new JMenu("Expression");
		menu.add(creater).setText("Add...");
		menu.add(remover).setText("Remove");
		menubar.add(menu);

		// functions presentations		
		list = new ListPane(this);
		list.setMinimumSize(new Dimension(100, 100));
		add(list);

		graph = new Grapher(this);
		add(graph);
	}
	
	void add(String expression) {
		add(FunctionFactory.createFunction(expression));
	}
	
	void add(Function function) {
		functions.addRow(new Object[] { function, Color.black }); 
	}
	public void remove(int index) {
		functions.removeRow(index);
	}

	public boolean isSelected(Function function) {
		for(int i = functions.getRowCount()-1; i >= 0; i--) {
 			Function f = (Function)functions.getValueAt(i, 0);
 			if(f == function) {
 				return list.isSelectedIndex(i);
 			}
 		}
 		return false;
	}
	
	void create_function() {
		String expression = JOptionPane.showInputDialog(FunctionPane.this, 
		                                                "Nouvelle expression :");
		try {
			add(expression);
		}
		catch(RuntimeException e) {
			JOptionPane.showMessageDialog(FunctionPane.this, 
			                              "\" " + expression + " \"\n" + 
			                              "Expression non valide !");
		}	
	}
	void remove_functions() {
		for(int i = functions.getRowCount()-1; i >= 0; i--) {
			if(list.isSelectedIndex(i)) {
				remove(i);
			}
		}
	}	
}
