package grapher.ui;

import java.awt.Dimension;
import java.awt.BorderLayout;
import javax.swing.JPanel;
import javax.swing.JTable;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.TableCellEditor;
import javax.swing.AbstractCellEditor;
import javax.swing.DefaultCellEditor;
import javax.swing.table.TableColumn;
import javax.swing.JTextField;
import javax.swing.JLabel;
import javax.swing.JButton;
import javax.swing.JColorChooser;
import javax.swing.JDialog;
import javax.swing.JOptionPane;
import java.awt.Component;
import java.awt.Color;

import javax.swing.JToolBar;

import java.awt.event.*;
import javax.swing.event.*;

import grapher.fc.*;


public class ListPane extends JPanel {
	protected JTable list;
	
	public ListPane(final FunctionPane functions) {
		super(new BorderLayout());
		
		list = new JTable(functions.functions) {
			public void valueChanged(ListSelectionEvent e) { 
				super.valueChanged(e);
				functions.repaint(); 
			}
			public void editingStopped(ChangeEvent e) {
				super.editingStopped(e);
				functions.repaint(); 
			}
		};

		TableColumn functions_column = list.getColumnModel().getColumn(0);
		functions_column.setCellEditor(new DefaultCellEditor(new JTextField()) {
			Function function = null;
			public boolean stopCellEditing() {
				String expression = (String)super.getCellEditorValue();
				try { function = FunctionFactory.createFunction(expression); }
				catch(RuntimeException e) {
					JOptionPane.showMessageDialog(functions, 
					                              "\" " + expression + " \"\n" + 
					                              "Expression non valide !");
					return false;
				}
				return super.stopCellEditing();
			}
			public Object getCellEditorValue() {
				return function;
			}
		});

		TableColumn colors_column = list.getColumnModel().getColumn(1);

		// custom renderer for color column
		class ColorRenderer extends JLabel 
		                    implements TableCellRenderer {
			public ColorRenderer() { setOpaque(true); }
			public Component getTableCellRendererComponent(JTable table, 
			                                               Object color,
			                                               boolean isSelected,
			                                               boolean hasFocus,
			                                               int row, int col) {
				setBackground((Color)color);
				return this;
			}
		}
		colors_column.setCellRenderer(new ColorRenderer());
		
		// custom editor for color column
		class ColorEditor extends AbstractCellEditor
		                  implements TableCellEditor,
		                             ActionListener {
			protected static final String EDIT = "edit";
			Color color;
			JButton button;
			JColorChooser chooser;
			JDialog dialog;
			
			public ColorEditor() {
				button = new JButton();
				button.setActionCommand(EDIT);
				button.addActionListener(this);
				button.setBorderPainted(false);
				chooser = new JColorChooser();
				dialog = JColorChooser.createDialog(button, "Pick a Color",
				                                    true, chooser,
				                                    this, null);
			}

			public void actionPerformed(ActionEvent e) {
				if(EDIT.equals(e.getActionCommand())) {
					button.setBackground(color);
					chooser.setColor(color);
					dialog.setVisible(true);
					fireEditingStopped();
				} else { //User pressed dialog's "OK" button
					color = chooser.getColor();
				}
			}

			public Object getCellEditorValue() { return color; }
			public Component getTableCellEditorComponent(JTable table,
			                                             Object value,
			                                             boolean isSelected,
			                                             int row, int col) {
				color = (Color)value;
				return button;
			}
		}
		ColorEditor color_editor = new ColorEditor();
		color_editor.addCellEditorListener(list);
		colors_column.setCellEditor(color_editor);
		colors_column.setMaxWidth(30);

		list.setPreferredSize(new Dimension(100, 100));
		add(list);
		
		JToolBar buttons = new JToolBar();
		buttons.setFloatable(false);
		buttons.add(functions.creater);
		buttons.add(functions.remover);
		add(buttons, BorderLayout.SOUTH);		
	}
	
	public boolean isSelectedIndex(int index) {
		return list.getSelectionModel().isSelectedIndex(index);
	}
}
